/******************************************************************************
  Copyright 2013 Tom Cornall

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.  
 ******************************************************************************/
#include <QApplication>
#include <QMessageBox>
#include <QInputDialog>
#include "DiagInterface.h"
#include "Frontend.h"

using namespace std;

/*
 *
 */
int main(int argc, char **argv)
{
    QApplication a(argc, argv);

    try
    {
        DiagInterface diag;

        /* find controller */
        vector<string> devPaths = diag.listDevPaths();
        string devPath;
        if (devPaths.empty())
        {
            QMessageBox::critical(NULL, "Error",
                    "No beamspring controller found");
            return 1;
        }
        else if (devPaths.size() == 1)
        {
            devPath = devPaths[0];
        }
        else
        {
            QStringList l;
            for (vector<string>::iterator i = devPaths.begin();
                    i != devPaths.end(); ++i)
                l.append(QString::fromStdString(*i));

            bool itemChosen;
            devPath = QInputDialog::getItem(NULL, "Choose keyboard", 
                    "Multiple beamspring controllers found, please choose one:",
                    l, 0, false, &itemChosen).toStdString();
            if (!itemChosen)
                return 1;
        }

        cerr << "path chosen: " << devPath << endl;
        diag.openDev(devPath);

        Frontend frontend(diag);
        frontend.show();

        return a.exec();
    }
    catch (std::exception &e)
    {
        QMessageBox::critical(NULL, "Error", QString::fromStdString(e.what()));
        return 1;
    }
}
